@@ -36,11 +36,6 @@ ListOfPortsType = types.Range(1, 65535)
 
 LOG = logging.getLogger(__name__)
 UNKNOWN_SERVICE_ID = 'unknown-service-id-error'
-HEAT_REMOVAL_DEPRECATION_WARNING = _('Support for heat templates in Trove is '
-                                     'scheduled for removal. You will no '
-                                     'longer be able to provide a heat '
-                                     'template to Trove for the provisioning '
-                                     'of resources.')
 
 path_opts = [
     cfg.StrOpt('pybasedir',
@@ -219,7 +214,7 @@ common_opts = [
                     'unreachable'),
     cfg.IntOpt('num_tries', default=3,
                help='Number of times to check if a volume exists.'),
-    cfg.StrOpt('volume_fstype', default='ext3',
+    cfg.StrOpt('volume_fstype', default='ext4',
                choices=['ext3', 'ext4', 'xfs'],
                help='File system type used to format a volume.'),
     cfg.StrOpt('cinder_volume_type', default=None,
@@ -1469,7 +1464,7 @@ rpcapi_cap_opts = [
         'taskmanager', default='latest',
         help='Set a version cap for messages sent to taskmanager services'),
     cfg.StrOpt(
-        'guestagent', default='epoxy',
+        'guestagent', default='flamingo',
         help='Set a version cap for messages sent to guestagent services'),
     cfg.StrOpt(
         'conductor', default='latest',
